import type {RPCCallPayload, Participant} from '@pexip/plugin-api';
import {registerPlugin} from '@pexip/plugin-api';

const uiState: RPCCallPayload<'ui:button:add'> = {
    position: 'toolbar',
    icon: 'IconPlayRound',
    tooltip: 'Record',
    roles: ['chair'],
};

const plugin = await registerPlugin({id: 'recording-plugin', version: 0});

let recorder: Participant | undefined = undefined;

const btn = await plugin.ui.addButton(uiState).catch(e => {
    console.warn(e);
});

const onBtnClick = async () => {
    if (recorder) {
        await stopRecording();
    } else {
        const input = await plugin.ui.showForm({
            title: 'Start recording',
            description:
                'The recording address should be a valid RTMPS address',
            form: {
                elements: {
                    recordingAddress: {
                        name: 'Recording address',
                        type: 'text',
                    },
                },
                submitBtnTitle: 'Start recording',
            },
        });
        const recordingAddress = input.recordingAddress;
        if (recordingAddress) {
            await startRecording(recordingAddress);
        }
    }
};
btn?.onClick.add(onBtnClick);

const onParticipantDisconnect = () => stopRecording(true);

const startRecording = async (recordingAddress: string) => {
    try {
        recorder = await plugin.conference.dialOut({
            destination: recordingAddress,
            streaming: 'yes',
            role: 'GUEST',
        });
        recorder.onDisconnect.add(onParticipantDisconnect);
        uiState.icon = 'IconStopRound';
        uiState.tooltip = 'Stop recording';
        await btn?.update(uiState);
    } catch (e) {
        console.warn(e);
    }
};
const stopRecording = async (externalDisconnect = false) => {
    if (!recorder) {
        return;
    }
    const updateState = async () => {
        recorder = undefined;
        uiState.icon = 'IconPlayRound';
        uiState.tooltip = 'Record';
        await btn?.update(uiState);
    };

    void plugin.ui.showToast({message: 'Stopping recorder'});
    if (!externalDisconnect) {
        const response = await recorder.disconnect();
        if (response?.data.status === 'success') {
            await updateState();
        }
    } else {
        await updateState();
    }
};
